/* This file was generated by upb_generator from the input file:
 *
 *     envoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_EXTENSIONS_GRPC_SERVICE_CHANNEL_CREDENTIALS_XDS_V3_XDS_CREDENTIALS_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_EXTENSIONS_GRPC_SERVICE_CHANNEL_CREDENTIALS_XDS_V3_XDS_CREDENTIALS_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__extensions__grpc_0service__channel_0credentials__xds__v3__XdsCredentials_msg_init;
extern const upb_MiniTable* envoy__extensions__grpc_0service__channel_0credentials__xds__v3__XdsCredentials_msg_init_ptr;

extern const upb_MiniTableFile envoy_extensions_grpc_service_channel_credentials_xds_v3_xds_credentials_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_EXTENSIONS_GRPC_SERVICE_CHANNEL_CREDENTIALS_XDS_V3_XDS_CREDENTIALS_PROTO_UPB_H__UPB_MINITABLE_H_ */
