/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/v2/promise.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */



#ifndef SRC_PROTO_GRPC_CHANNELZ_V2_PROMISE_PROTO_UPB_H__UPBDEFS_H_
#define SRC_PROTO_GRPC_CHANNELZ_V2_PROMISE_PROTO_UPB_H__UPBDEFS_H_

#include "upb/reflection/def.h"
#include "upb/reflection/internal/def_pool.h"

#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern _upb_DefPool_Init src_proto_grpc_channelz_v2_promise_proto_upbdefinit;

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_Custom_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.Custom");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_If_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.If");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_Loop_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.Loop");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_ForEach_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.ForEach");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_Race_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.Race");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_SeqStep_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.SeqStep");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_Seq_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.Seq");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_JoinBranch_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.JoinBranch");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_Join_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.Join");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v2_Promise_Map_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_v2_promise_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v2.Promise.Map");
}


#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* SRC_PROTO_GRPC_CHANNELZ_V2_PROMISE_PROTO_UPB_H__UPBDEFS_H_ */
