/* This file was generated by upb_generator from the input file:
 *
 *     cel/expr/checked.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */



#ifndef CEL_EXPR_CHECKED_PROTO_UPB_H__UPBDEFS_H_
#define CEL_EXPR_CHECKED_PROTO_UPB_H__UPBDEFS_H_

#include "upb/reflection/def.h"
#include "upb/reflection/internal/def_pool.h"

#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern _upb_DefPool_Init cel_expr_checked_proto_upbdefinit;

UPB_INLINE const upb_MessageDef *cel_expr_CheckedExpr_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.CheckedExpr");
}

UPB_INLINE const upb_MessageDef *cel_expr_CheckedExpr_ReferenceMapEntry_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.CheckedExpr.ReferenceMapEntry");
}

UPB_INLINE const upb_MessageDef *cel_expr_CheckedExpr_TypeMapEntry_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.CheckedExpr.TypeMapEntry");
}

UPB_INLINE const upb_MessageDef *cel_expr_Type_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Type");
}

UPB_INLINE const upb_MessageDef *cel_expr_Type_ListType_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Type.ListType");
}

UPB_INLINE const upb_MessageDef *cel_expr_Type_MapType_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Type.MapType");
}

UPB_INLINE const upb_MessageDef *cel_expr_Type_FunctionType_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Type.FunctionType");
}

UPB_INLINE const upb_MessageDef *cel_expr_Type_AbstractType_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Type.AbstractType");
}

UPB_INLINE const upb_MessageDef *cel_expr_Decl_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Decl");
}

UPB_INLINE const upb_MessageDef *cel_expr_Decl_IdentDecl_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Decl.IdentDecl");
}

UPB_INLINE const upb_MessageDef *cel_expr_Decl_FunctionDecl_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Decl.FunctionDecl");
}

UPB_INLINE const upb_MessageDef *cel_expr_Decl_FunctionDecl_Overload_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Decl.FunctionDecl.Overload");
}

UPB_INLINE const upb_MessageDef *cel_expr_Reference_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &cel_expr_checked_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "cel.expr.Reference");
}


#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* CEL_EXPR_CHECKED_PROTO_UPB_H__UPBDEFS_H_ */
